----------------------------------------------------------------
-- Color senders & receivers management
----------------------------------------------------------------
local addon = KiwiPlates

local wipe = wipe
local next = next
local unpack = unpack
local tdelete = addon.tdelete

local meta = { __index = function(t,k) local v={}; t[k]=v; return v end }
local colors     = setmetatable( {}, meta )
local priorities = setmetatable( {}, meta )
local senders    = setmetatable( {}, meta )
local receivers  = setmetatable( {}, meta )
local SendColor  = setmetatable( {}, { __index = function() return function() end end } )

local function UpdateReceiver(kPlate, receiver, object)
	local senders    = senders[receiver]
	local priorities = priorities[kPlate]
	local colors     = colors[kPlate]
	for i=1,#senders do
		local sender = senders[i]
		local color  = colors[sender]
		if color then
			priorities[receiver] = senders[sender]
			object:SetColor( color[1], color[2], color[3], color[4] )
			return
		end
	end
	priorities[receiver] = 0
	object:SetColor( unpack(object.defaultColor or addon.defaultColor) )
end

local function CreateSR(sender, receiver, priority)
	return function(kPlate, color)
		local object = kPlate[receiver]
		if object and object.SetColor then
			colors[kPlate][sender] = color or nil
			if color then
				local priorities = priorities[kPlate]
				if priority >= (priorities[receiver] or 0) then
					priorities[receiver] = priority
					object:SetColor( color[1], color[2], color[3], color[4] )
				end
			else
				UpdateReceiver(kPlate, receiver, object)
			end	
		end	
	end	
end

local function CreateMR(sender, receivers, priority)
	return function(kPlate, color)
		colors[kPlate][sender] = color or nil
		if color then
			local priorities = priorities[kPlate]
			for i=1,#receivers do
				local receiver = receivers[i]
				local object   = kPlate[receiver]
				if object and object.SetColor and priority >= (priorities[receiver] or 0) then
					priorities[receiver] = priority
					object:SetColor( color[1], color[2], color[3], color[4] )
				end
			end	
		else
			for i=1,#receivers do
				local receiver = receivers[i]
				local object = kPlate[receiver]
				if object and object.SetColor then 
					UpdateReceiver(kPlate, receiver, object)	
				end	
			end	
		end	
	end
end

function addon.UnregisterColorSender( sender )
	if SendColor[sender] then
		wipe( receivers[sender] )
		for _,senders in next,senders do
			if senders[sender] then
				senders[sender] = nil
				tdelete(senders, sender)
			end
		end
		SendColor[sender] = nil
	end	
end

function addon.RegisterColorSender( sender, senderReceivers, priority )
	addon.UnregisterColorSender(sender)
	local receivers = receivers[sender]	
	if senderReceivers and #senderReceivers>0 then
		for _,receiver in next,senderReceivers do
			local senders = senders[receiver]
			senders[#senders+1] = sender
			senders[sender]     = priority
			table.sort( senders, function(a,b) return senders[a]>senders[b] end )
			tinsert( receivers, receiver )		
		end
		SendColor[sender] = (#receivers>1 and CreateMR(sender, receivers, priority)) or CreateSR(sender, receivers[1], priority)
	end	
end

function addon.UpdateColorReceiver(kPlate, receiver)
	local object = kPlate[receiver]
	if object and object.SetColor then
		UpdateReceiver(kPlate, receiver, object)
	end	
end

function addon.PlateResetColors(kPlate)
	wipe( colors[kPlate] )
	wipe( priorities[kPlate] )
end

function addon.ResetAllPlateColors()	
	local kaPlates = addon.kaPlates
	for i=1,#kaPlates do
		for _,sender in next,SendColor do
			sender(kaPlates[i])
		end
	end
end

addon.SendColor = SendColor
