local media = LibStub("LibSharedMedia-3.0", true)
local addon  = KiwiPlates
local widget = addon:newWidget( 'Auras', { 'auras' } )
local Notify = addon.Notify

local auraExpiration = addon.auraExpiration
local auraCount      = addon.auraCount
local auraIcon       = addon.auraIcon

local wipe = wipe
local next = next
local unpack = unpack
local tremove = tremove
local C_Timer = C_Timer
local GetTime = GetTime

local aurasDB
local savedAuras = {}
local activeAuras = {}
local activeCount = 0
local UpdateAuras

local function AuraCreate()
	local Aura = CreateFrame('Frame')
	Aura.Back = Aura:CreateTexture(nil, 'BACKGROUND')
	Aura.Back:SetAllPoints()
	Aura.TimeText  = Aura:CreateFontString(nil, "ARTWORK")
	Aura.TimeText:SetPoint('CENTER')
	Aura.CountText = Aura:CreateFontString(nil, "ARTWORK")
	Aura.CountText:SetPoint('CENTER')
	local kIcon = Aura:CreateTexture(nil, 'BORDER')
	kIcon:SetTexCoord( 0.15, 0.85, 0.15, 0.85 )
	kIcon:SetPoint('CENTER')
	Aura.kIcon = kIcon	
	return Aura
end

local function AuraLayout(Aura, dba)
	local size2 = (dba.borderSize or 0)*2
	local width = dba.auraWidth-size2
	local height = dba.auraHeight-size2
	Aura:SetSize( dba.auraWidth, dba.auraHeight)
	Aura.kIcon:SetSize( width, height )	
	if dba.filename then
		Aura.Back:SetTexture( dba.filename )
		Aura.Back:SetDrawLayer( dba.layer or 'ARTWORK', dba.sublayer or 4)
	else
		Aura.Back:SetDrawLayer( 'BACKGROUND', 0)
		Aura.Back:SetTexture( unpack(dba.borderColor) )
	end
	local db = dba.TimeText
	local kText = Aura.TimeText
	kText:SetFont( media:Fetch('font', db.fontName), db.fontSize, db.fontFlags)
	kText:SetTextColor( unpack(db.fontColor) )
	kText:SetJustifyH( db.JustifyH )
	kText:SetJustifyV( db.JustifyV )	
	kText:SetSize(width+db.AdjustX,height+db.AdjustY)
	local db = dba.CountText
	local kText = Aura.CountText
	kText:SetFont( media:Fetch('font', db.fontName), db.fontSize, db.fontFlags)
	kText:SetTextColor( unpack(db.fontColor) )
	kText:SetJustifyH( db.JustifyH )
	kText:SetJustifyV( db.JustifyV )
	kText:SetSize(width+db.AdjustX,height+db.AdjustY)	
end

local function AuraFactoryGet(kPlate, Auras, placeAura, auraId)
	local Aura = tremove(savedAuras) or AuraCreate(kPlate)
	--
	Auras[#Auras+1] = Aura
	Auras[auraId]   = Aura
	--
	Aura:SetParent(kPlate)
	Aura:SetFrameLevel( kPlate:GetFrameLevel() )
	Aura.kPlate = kPlate	
	Aura.auraId = auraId
	--
	local skinName = kPlate.skinName
	if Aura.skinName ~= skinName then
		AuraLayout(Aura, kPlate.db.Auras)
		Aura.skinName = skinName
	end
	--
	if placeAura then
		local db = kPlate.db.Auras
		Aura:ClearAllPoints()
		Aura:SetPoint( 'CENTER', db.left + (#Auras-1) * (db.auraWidth + db.auraSpacing), db.top )
	end
	--
	Aura.kIcon:SetTexture( auraIcon[auraId] )
	Aura:Show()
	--
	activeAuras[Aura] = Aura.TimeText
	activeCount = activeCount + 1
	if activeCount==1 then 
		C_Timer.After( 0.1, UpdateAuras )
	end
	return Aura
end

local function AuraFactoryPut(Aura)
	activeCount = activeCount - 1
	activeAuras[Aura] = nil
	savedAuras[#savedAuras+1] = Aura
	Aura:SetParent(nil)
	Aura:Hide()
end

local function AuraSetValues(Aura, expiration, count)
	Aura.expiration = expiration
	Aura.count = count
	Aura.CountText:SetText( count>1 and count or '')	
	if expiration>0 then
		Aura.TimeText:SetFormattedText( "%d", expiration-GetTime() )
	else
		Aura.TimeText:SetText('')
	end	
end

local function PlateAuraUpdate(kPlate, auraId, expiration, count)
	local Auras = kPlate.Auras
	if Auras then
		local Aura = Auras[auraId] or AuraFactoryGet(kPlate, Auras, true, auraId)
		AuraSetValues(Aura, expiration, count)
	end	
end

local function PlateAuraRemove(kPlate, auraId)
	local Auras = kPlate.Auras
	if Auras then
		local Aura = Auras[auraId]
		if Aura then
			AuraFactoryPut(Aura)
			Auras[auraId] = nil
			for i=1,#Auras do
				if Auras[i] == Aura then
					tremove( Auras, i )
					local db = kPlate.db.Auras
					for index=i,#Auras do
						local Aura = Auras[index]
						Aura:ClearAllPoints()
						Aura:SetPoint( 'CENTER', db.left + (index-1) * (db.auraWidth + db.auraSpacing), db.top )
					end
					return
				end
			end
		end	
	end	
end

local function PlateShow(kPlate, bPlate)
	local Auras = kPlate.Auras
	if Auras then
		local guid = bPlate.guid
		if guid then
			for auraId,expiration in next,auraExpiration[guid] do
				local Aura = AuraFactoryGet(kPlate, Auras, true, auraId )
				AuraSetValues( Aura, expiration, auraCount[guid][auraId] )
			end
		end
	end	
end

local function PlateHide(kPlate)
	local Auras = kPlate.Auras
	if Auras then
		for i=1,#Auras do
			AuraFactoryPut(Auras[i])
		end
		wipe(Auras)
	end
end

local function PlateCreate(kPlate)
	if kPlate.db.Auras then
		kPlate.Auras = kPlate.Auras or {}
	end	
end

local function PlateFrameLevelUpdate(kPlate)
	local Auras = kPlate.Auras
	if Auras and #Auras>0 then
		local level = kPlate:GetFrameLevel()
		for i=#Auras,1,-1 do
			Auras[i]:SetFrameLevel( level )
		end
	end
end

function UpdateAuras()
	if activeCount>0 then
		local time = GetTime()
		for Aura, TimeText in next,activeAuras do
			local expiration = Aura.expiration
			if expiration>0 then
				local timeLeft = expiration - time
				if timeLeft>0 then
					if timeLeft>=1 then
						TimeText:SetFormattedText( '%d', timeLeft )
					else
						TimeText:SetFormattedText( '|cffff0000.%d', timeLeft*10 )
					end
				else
					PlateAuraRemove(Aura.kPlate, Aura.auraId)
				end
			end
		end
		if activeCount>0 then
			C_Timer.After( 0.1, UpdateAuras)
		end			
	end
end

function widget:Layout(skin)
	for _,Aura in next, savedAuras do
		if skin.name == Aura.skinName then
			Aura.skinName = nil
		end
	end	
	for _,kPlate in next, addon.kvPlates do
		if kPlate.Auras and skin.name == kPlate.skinName then
			for _,Aura in next, kPlate.Auras do
				Aura.SkinName = nil
			end
		end
	end	
end

function widget:OnEnable(addon, themedb)
	if themedb.Auras then
		addon.RegisterNotify( 'PlateCreate', PlateCreate )
		addon.RegisterNotify( 'PlateShow', PlateShow )
		addon.RegisterNotify( 'PlateHide', PlateHide )	
		addon.RegisterNotify( 'PlateAurasUpdate', PlateShow)	
		addon.RegisterNotify( 'PlateAuraUpdate', PlateAuraUpdate )
		addon.RegisterNotify( 'PlateAuraRemove', PlateAuraRemove )
		addon.RegisterNotify( 'PlateFrameLevelUpdate', PlateFrameLevelUpdate )
		for _,kPlate in next,addon.kaPlates do PlateCreate(kPlate) end
	end	
end

function widget:OnDisable(addon)
	addon.UnregisterNotify( 'PlateCreate', PlateCreate )
	addon.UnregisterNotify( 'PlateShow', PlateShow )
	addon.UnregisterNotify( 'PlateHide', PlateHide )	
	addon.UnregisterNotify( 'PlateAurasUpdate', PlateShow)		
	addon.UnregisterNotify( 'PlateAuraUpdate', PlateAuraUpdate )
	addon.UnregisterNotify( 'PlateAuraRemove', PlateAuraRemove )
	addon.UnregisterNotify( 'PlateFrameLevelUpdate', PlateFrameLevelUpdate )
end
