
local addon  = KiwiPlates
local widget = addon:newWidget( 'Mouseover', { 'mouseover' } )
local Notify = addon.Notify
local SendColor = addon.SendColor
local PlateSendToBack   = addon.PlateSendToBack
local PlateBringToFront = addon.PlateBringToFront

local unpack = unpack
local defaultColor = {1,1,1,0.2}

local ktHighlight, ktEnabled

local function SetHighlightTexture(kPlate)
	local skinName = kPlate.skinName
	local tex = ktHighlight[skinName]
	if not tex then
		tex = ktHighlight:CreateTexture(nil, 'OVERLAY')
		local db = kPlate.db.Mouseover
		if db then
			tex:SetTexture( db.filename )
			tex:SetVertexColor( unpack(db.color or defaultColor) )
			if db.coords then tex:SetTexCoord( unpack(db.coords) ) end
			if db.points then 
				tex:SetPoint('CENTER', db.points[1], db.points[2])
				tex:SetSize(db.points[3], db.points[4] )
			else	
				tex:SetAllPoints()				
			end	
		else
			tex:SetTexture(1,1,1,0.2)
			tex:SetAllPoints()				
		end
		tex:SetBlendMode('ADD')
		ktHighlight[skinName] = tex
	end
	if tex~=ktEnabled then
		if ktEnabled then ktEnabled:Hide() end
		ktEnabled = tex
		ktEnabled:Show()
	end
end

local function MouseEnter(kPlate, bPlate)
	if not bPlate.isTarget then
		local db = addon.theme.Mouseover
		if db then
			PlateBringToFront(kPlate, bPlate, 'Mouseover', 127)
			if not kPlate.db.Plate.disableHighlight then
				SetHighlightTexture(kPlate)
				ktHighlight:SetParent( kPlate )
				ktHighlight:SetFrameLevel( kPlate:GetFrameLevel() )	
				ktHighlight:SetAllPoints( kPlate )
				ktHighlight:Show()
			end	
			if db.glowEnabled then
				SendColor.Mouseover( kPlate , db.glowColor )
			end
		end	
	end	
end

local function MouseLeave(kPlate)
	local db = addon.theme.Mouseover
	if db then
		PlateSendToBack(kPlate, kPlate.bPlate, 'Mouseover' )
		ktHighlight:Hide()
		if db.glowEnabled then
			SendColor.Mouseover( kPlate )
		end
	end	
end

function widget:OnEnable(addon, theme)
	local db = theme.Mouseover
	ktHighlight = ktHighlight or CreateFrame( 'Frame' )
	ktHighlight:Hide()
	addon.RegisterNotify( 'PlateMouseEnter', MouseEnter )
	addon.RegisterNotify( 'PlateMouseLeave', MouseLeave )
	addon.RegisterColorSender( 'Mouseover', db.colorReceivers, db.colorPriority or 5)
end

function widget:OnDisable(addon)
	addon.UnregisterNotify( 'PlateMouseEnter', Update )
	addon.UnregisterNotify( 'PlateMouseLeave', Update )
	addon.UnregisterColorSender( 'Mouseover' )	
end
