
local addon      = KiwiPlates
local widget     = addon:newWidget( 'Target', { 'target' } )
local SendColor  = addon.SendColor
local kvPlates   = addon.kvPlates
local PlatesSetAlpha    = addon.PlatesSetAlpha
local PlateSendToBack   = addon.PlateSendToBack
local PlateBringToFront = addon.PlateBringToFront

local defaultColor = {1,1,1,0.2}

local targetDB
local opaqueNT
local kPlateTarget
local ktHighlight, ktEnabled

local function SetHighlightTexture(kPlate)
	local skinName = kPlate.skinName
	local tex = ktHighlight[skinName]
	if not tex then
		tex = ktHighlight:CreateTexture(nil, 'OVERLAY')
		local db = kPlate.db.Target
		if db then
			tex:SetTexture( db.filename )
			tex:SetVertexColor( unpack(db.color or defaultColor) )
			if db.coords then tex:SetTexCoord( unpack(db.coords) ) end
		else
			tex:SetTexture(1,1,1,0.15)
		end
		tex:SetBlendMode('ADD')
		tex:SetAllPoints()
		ktHighlight[skinName] = tex
	end
	if tex~=ktEnabled then
		if ktEnabled then ktEnabled:Hide() end
		ktEnabled = tex
		ktEnabled:Show()
	end
end

local function Update(kPlate,bPlate)
	if bPlate.isTarget then
		kPlateTarget = kPlate	
		PlateBringToFront( kPlate, bPlate, 'Target', 128 )
		if targetDB.glowEnabled then
			SendColor.Target( kPlate, targetDB.glowColor )
		end
		if not kPlate.db.Plate.disableHighlight then
			SetHighlightTexture( kPlate )
			ktHighlight:SetParent( kPlate )
			ktHighlight:SetFrameLevel( kPlate:GetFrameLevel() )	
			ktHighlight:SetAllPoints( kPlate )
			ktHighlight:Show()
		end	
	else
		PlateSendToBack( kPlate, bPlate, 'Target', 128 )
		if targetDB.glowEnabled then
			SendColor.Target( kPlate )
		end
	end
end

local function Hide(kPlate)
	if kPlate == kPlateTarget then
		ktHighlight:Hide()
		if targetDB.glowEnabled then
			SendColor.Target( kPlate )
		end
		kPlateTarget = nil
	end
end

local function TargetChanged( newTarget, oldTarget )
	if opaqueNT and ( not newTarget ~= not oldTarget ) then
		PlatesSetAlpha(newTarget and addon.theme.Plates.alpha or 1)
	end
	if not newTarget then
		ktHighlight:Hide()
	end
end

function widget:OnEnable(addon, theme )
	local db = theme.Target
	targetDB = db
	opaqueNT = db.opaqueWhenNoTarget
	ktHighlight = ktHighlight or CreateFrame( 'Frame' )
	addon.RegisterNotify( 'PlateShow', Update )
	addon.RegisterNotify( 'PlateHide', Hide )	
	addon.RegisterNotify( 'PlateTargetUpdate', Update )
	if opaqueNT or ktGlow then addon.RegisterNotify( 'TargetChanged', TargetChanged ) end	
	addon.RegisterColorSender( 'Target', db.colorReceivers, db.colorPriority or 5)
	addon.PlatesSetAlpha( addon.TargetGUID and addon.theme.Plates.alpha or 1 )
end

function widget:OnDisable(addon)
	addon.UnregisterNotify( 'PlateShow', Update )
	addon.UnregisterNotify( 'PlateHide', Hide )		
	addon.UnregisterNotify( 'PlateTargetUpdate', Update )
	if opaqueNT or ktGlow then addon.UnregisterNotify( 'TargetChanged', TargetChanged )	end	
	addon.UnregisterColorSender( 'Target')
end

