
local addon  = KiwiPlates
local status = addon:newStatus( 'threat', { 'AceEvent-3.0' } )
local PlateSkinRules = addon.PlateSkinRules
local Notify = addon.Notify

local next = next
local UnitIsUnit = UnitIsUnit
local UnitThreatSituation = UnitThreatSituation

local bvPlates = addon.bvPlates
local RegisterNotify = addon.RegisterNotify
local UnregisterNotify = addon.UnregisterNotify

local offtank_enabled
local offtank_active
local offtankByUnit = addon.offtankByUnit
local offtankByGuid = addon.offtankByGuid
local offthreats = {}
local targets = setmetatable( {}, { __index = function(t,k) local v = k..'target'; t[k] = v; return v end } )
	
local threatLast, registered

local combat_events = { 
	SPELL_DAMAGE = true, SPELL_MISSED = true, 
	SWING_DAMAGE = true, SWING_MISSED = true, 
	RANGE_DAMAGE = true, RANGE_MISSED = true,
}

--

local function Update(bPlate)
	local bAggro = bPlate.bAggroWarningTexture
	if bAggro:IsVisible() then
 		local r, g, b = bAggro:GetVertexColor()
		local threat = (r>0.9 and g+b<0.1) and 'tanked' or 'aggroed'
		bPlate.vThreat = threat
		return threat
	elseif offtank_active and bPlate.guid and offthreats[bPlate.guid] then
		bPlate.vThreat = 'offtanked'
		return 'offtanked'

	end
	bPlate.vThreat  = nil
end

local function UpdateAllPlates()
	for i=#bvPlates,1,-1 do
		local bPlate = bvPlates[i]
		local threat = bPlate.vThreat
		if Update(bPlate) ~= threat then
			if not (PlateSkinRules.threat and addon.ReSkinPlate(bPlate)) then
				Notify.PlateThreatUpdate( bPlate.kPlate, bPlate )
			end	
		end
	end
end

local function ClearAllPlates()
	for i=#bvPlates,1,-1 do 
		local bPlate = bvPlates[i]
		if bPlate.vThreat then
			bPlate.vThreat = nil
			if not (PlateSkinRules.threat and addon.ReSkinPlate(bPlate)) then
				Notify.PlateThreatUpdate( bPlate.kPlate, bPlate )
			end	
		end
	end
end

local function EnableThreatUpdate(enabled)
	if not registered ~= not enabled then
		registered = enabled
		if enabled then
			RegisterNotify( 'Update', UpdateAllPlates )
		else
			UnregisterNotify( 'Update', UpdateAllPlates )
			ClearAllPlates()			
		end	
	end
end

local function UnitThreatEvent(_,unit)
	if unit then
		if offtank_active then
			local target = targets[unit]
			local guid = UnitGUID(target)
			if guid and (UnitThreatSituation(unit, target) or 0)>1 then
				offthreats[guid] = offtankByUnit[unit]
			end
		elseif UnitIsUnit(unit,'player') then
			local threat = UnitThreatSituation(unit)
			threat = not (threat==nil or threat==0)
			if threat~=threatLast then
				threatLast = threat
				EnableThreatUpdate(threat)
			end
		end	
	end	
end

local function CombatLogEvent(_, timestamp, event, _, srcGUID, _, _, _, dstGUID)
	if combat_events[event] then
		offthreats[srcGUID or 0] = offtankByGuid[dstGUID or 0]
	end
end

local function PlayerRegenDisabled()
	offtank_active = next(offtankByUnit)
	if offtank_active then
		status:RegisterEvent('COMBAT_LOG_EVENT_UNFILTERED', CombatLogEvent)
		EnableThreatUpdate(true)
	end	
end

local function PlayerRegenEnabled()
	offtank_active = false
	wipe(offthreats)
	EnableThreatUpdate(false)
	status:UnregisterEvent('COMBAT_LOG_EVENT_UNFILTERED')
end

function status:EnableOffTankThreat(enabled)
	if not offtank_enabled ~= not enabled then
		offtank_enabled = enabled
		offtank_active  = false
		if enabled then
			self:RegisterEvent('PLAYER_REGEN_ENABLED', PlayerRegenEnabled)
			self:RegisterEvent('PLAYER_REGEN_DISABLED', PlayerRegenDisabled)
		else
			self:UnregisterEvent('PLAYER_REGEN_ENABLED')
			self:UnregisterEvent('PLAYER_REGEN_DISABLED')
		end
	end	
end

function status:OnEnable(addon, dbx)
	addon.RegisterNotify( 'bPlateShow', Update )
	self:RegisterEvent( 'UNIT_THREAT_SITUATION_UPDATE', UnitThreatEvent ) 
	UnitThreatEvent(nil,'player')
end

function status:OnDisable(addon)
	threatLast = nil
	EnableThreatUpdate(false)	
	self:EnableOffTankThreat(false)
	self:UnregisterEvent( 'UNIT_THREAT_SITUATION_UPDATE' )	
	addon.UnregisterNotify( 'bPlateShow', Update )
end
