local media = LibStub("LibSharedMedia-3.0", true)
local widget = KiwiPlates:newWidget( 'HealthText', { 'health' } )

local unpack = unpack

local formats = {
	{ '%d',         '%dK',        '%.1fM'        },
	{ '%d | %d%%',  '%dK | %d%%', '%.1fM | %d%%' },
	{ '%d%% | %d',  '%d%% | %dK', '%d%% | %.1fM' },	
}	

local textFormat

local function Update(kPlate,bPlate)
	local HealthText = kPlate.HealthText
	if HealthText then
		local db       = kPlate.db.HealthText
		local vHealthP = bPlate.vHealth
		if vHealthP<=0.999 or (not db.hideAtFullHealth) then
			local vHealth  = bPlate.vMaxHealth
			local index    = textFormat or db.textFormat or 1
			if index>1 and vHealth then
				local fmt
				if vHealth>1000000 then
					vHealth = vHealthP * vHealth / 1000000
					fmt = formats[index-1][3]
				elseif vHealth>10000 then
					vHealth = vHealthP * vHealth / 1000
					fmt = formats[index-1][2]
				else
					vHealth = vHealthP * vHealth
					fmt = formats[index-1][1]
				end
				vHealthP = vHealthP * 100
				if index == 4 then
					vHealth,vHealthP = vHealthP, vHealth
				end
				HealthText:SetFormattedText( fmt, vHealth, vHealthP )
			else
				HealthText:SetFormattedText( "%d%%", vHealthP*100 )
			end
			HealthText:SetShown(true)
		else
			HealthText:SetShown(false)
		end		
	end	
end

local function PlateGuidStored(bPlate)
	if bPlate.kPlate then
		Update(bPlate.kPlate, bPlate)
	end	
end

local function Create(kPlate)
	local db = kPlate.db.HealthText
	if db then
		local HealthText = kPlate.HealthText or kPlate:CreateFontString(nil, "ARTWORK")
		HealthText.defaultColor = db.fontColor
		HealthText:SetFont( media:Fetch('font', db.fontName), db.fontSize, db.fontFlags)
		HealthText:SetTextColor( unpack(db.fontColor) )
		HealthText:SetSize( db.width or kPlate.width, db.height or kPlate.height )
		HealthText:SetJustifyH( db.JustifyH )
		HealthText:SetJustifyV( db.JustifyV )	
		HealthText:SetPoint( 'CENTER', db.left or 0, db.top or 0 )
		HealthText.SetColor = db.acceptColors and HealthText.SetTextColor or nil
		kPlate.HealthText = HealthText
	end	
end

function widget:OnEnable(addon, theme)
	textFormat =  theme.HealthText and theme.HealthText.textFormat
	addon.RegisterNotify( 'PlateCreate', Create )
	addon.RegisterNotify( 'PlateShow', Update )
	addon.RegisterNotify( 'PlateHealthUpdate', Update )
	addon.RegisterNotify( 'PlateGuidStored', PlateGuidStored )
end

function widget:OnDisable(addon)
	addon.UnregisterNotify( 'PlateCreate', Create )
	addon.UnregisterNotify( 'PlateShow', Update )
	addon.UnregisterNotify( 'PlateHealthUpdate', Update )
	addon.UnregisterNotify( 'PlateGuidStored', PlateGuidStored )	
end
