----------------------------------------------------------------
--
----------------------------------------------------------------

local addon   = KiwiPlates
local tdelete = addon.tdelete

local categories = {}
local modules = {}
local widgets = {}
local widget_defaults = {}
local module_defaults = {}

local options = {
	options = {
		name = "KiwiPlates",
		type = "group",
        get = "Get",
        set = "Set",
		childGroups = "tab",		
		args = {},
	},
	widget_defaults = widget_defaults,
	module_defaults = module_defaults,
}

----------------------------------------------------------------
--
----------------------------------------------------------------

local handler_proto = {}

function handler_proto:search_db(info)
	local function search(index,db)
		if index>0 then
			local key = info[index]
			if  key ~= self.dbname then
				db = search(index-1,db)
				return #key>2 and db[key] or db
			end
			return db
		end	
	end
	return search(#info-1, self.db)
end

local defaultColor = { 1,1,1,1 }
function handler_proto:Get(info, itemkey)
	local db  = self:search_db(info)
	local key = info[#info]
	if info.type == 'multiselect' then
		if db[key] then
			if #db[key]>0 then
				return not not tContains(db[key], itemkey)
			else
				return db[key][itemkey]
			end			
		end	
	else
		local value = db[ key ]
		if info.type == 'color' then
			return unpack(value or defaultColor)
		else	
			return value
		end
	end	
end

function handler_proto:Set(info, value, ...)
	local db  = self:search_db(info)
	local key = info[#info]
	if info.type == 'multiselect' then
		if not db[key] or #db[key]>0 then
			if (...) then
				if db[key] then 
					tinsert( db[key], value )
				else
					db[key] = { value }
				end
			else
				tdelete( db[key], value )
			end
		else
			db[key][value] = not not (...)
		end		
	elseif info.type == 'color' then
		local color = { value, ... }
		if info.option.hasAlpha == false then
			color[4] = nil -- Special case , remove alpha value
		end	
		db[ key ] = color
	elseif self.skin and value==0 and info.type == 'range' and (key == 'left' or key =='top' or key == 'width' or key == 'height') then
		db[ key ] = false
	else
		db[ key ] = value
	end
	if self.OnChange then 
		self:OnChange(info) 
	end	
end

function options:newHandler( child, parent )
	return setmetatable( child or {}, { __index = parent or handler_proto }  )
end

----------------------------------------------------------------
--
----------------------------------------------------------------

function options:RegisterItemOptions( regTable, name, options, params )
	regTable[name] = {
		type   = "group", 
		order  = params and params.order or 10,	
		name   = params and params.inline and '' or name, 
		inline = params and params.inline,
		arg    = { options = options, handler = params and params.handler },
		args   = options,
	}
	return regTable[name]
end

function options:RegisterWidgetOptions( name, options, params )
	return self:RegisterItemOptions( widgets, name, options, params )
end

function options:RegisterModuleOptions( name, options, params )
	return self:RegisterItemOptions( modules, name, options, params )
end

function options:RegisterCategoryOptions( name, options, params )
	self.options.args[name] = self:RegisterItemOptions( categories, name, options, params )
end

----------------------------------------------------------------
--
----------------------------------------------------------------

function options:MakeItemOptions( regTable, name, db, regParent )
	local data = regTable[name]
	if (not data) and type(db)=='table' then
		data = self:RegisterItemOptions( regTable, name )
	end
	if data then
		data.handler        = self:newHandler( data.arg.handler, regParent and regParent.arg.handler )
		data.handler.dbname = name
		data.handler.db     = db
		local options = data.arg.options
		if type(options) == 'table' then
			data.args = options
		elseif type(options) == 'function' then	
			data.args = options(db) 
		else
			data.args = self:MakeAutomaticOptions(db, name)
		end	
	end	
	return data
end

function options:MakeCategoryOptions( name, db )
	return self:MakeItemOptions( categories, name, db)
end

function options:MakeModuleOptions( name, db )
	return self:MakeItemOptions( modules, name, db, categories.Themes )
end

function options:MakeWidgetOptions( name, db  )
	return self:MakeItemOptions( widgets, name, db, categories.Skins )
end

----------------------------------------------------------------
--
----------------------------------------------------------------

function options:RegisterModuleDefaults( name, def )
	module_defaults[name] = def
end

function options:GetModuleDefaults( name , copy)
	local defaults = module_defaults[name]
	if defaults then
		return copy and addon.CopyTable(defaults) or defaults
	end	
	return {}
end

----------------------------------------------------------------
--
----------------------------------------------------------------

function options:RegisterWidgetDefaults( name, def )
	widget_defaults[name] = def
end

function options:GetWidgetDefaults( name , copy)
	local defaults = widget_defaults[name]
	if defaults then
		return copy and addon.CopyTable(defaults) or defaults
	end	
end

----------------------------------------------------------------
--
----------------------------------------------------------------

function options:MakeOptions()
	local cat = self.options.args
	for key,value in pairs(cat) do
		cat[key] = self:MakeCategoryOptions(key)
	end
end

----------------------------------------------------------------
--
----------------------------------------------------------------

function options:Initialize()
	LibStub("AceConfig-3.0"):RegisterOptionsTable("KiwiPlates", self.options)
	self:MakeOptions()
	self.Initialize = nil
end

function options:OnChatCommand(input)
	if (LibStub("AceConfigDialog-3.0").OpenFrames["KiwiPlates"]) then
		LibStub("AceConfigDialog-3.0"):Close("KiwiPlates")
	else
		LibStub("AceConfigDialog-3.0"):Open("KiwiPlates")
	end
end

--{{
_G.KiwiPlatesOptions = options
--}}
